# Copyright (c) 2018 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from kolla_cli.tests.unit.common import KollaCliUnitTest


class TestUnit(KollaCliUnitTest):

    @mock.patch('cliff.lister.Lister.produce_output')
    @mock.patch('kolla_cli.api.client.ClientApi.service_get_all')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_service_list(self, _, mock_service_get_all, mock_cliff):
        # list all services, check that output is sorted properly
        servicename1 = 'service1'
        servicename2 = 'service2'
        childnames = ['child2', 'child1']
        fake_service1 = self.get_fake_service(servicename1,
                                              childnames=childnames)
        fake_service2 = self.get_fake_service(servicename2,
                                              childnames=childnames)
        mock_service_get_all.return_value = [fake_service2, fake_service1]
        ret = self.run_cli_command('service list')
        self.assertEqual(ret, 0)
        mock_service_get_all.assert_called_once_with()

        expected_childnames = '[child1,child2]'
        mock_cliff.assert_called_once_with(
            mock.ANY, ('Service', 'Children'),
            [(servicename1, expected_childnames),
             (servicename2, expected_childnames)
             ])

    @mock.patch('kolla_cli.api.client.ClientApi.service_get_all')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_service_grouplist(self, _, mock_service_get_all):
        # list all services with their groups
        servicename = 'service1'
        groupname = 'group1'
        fake_service = self.get_fake_service(servicename,
                                             groupnames=[groupname])
        mock_service_get_all.return_value = [fake_service]
        ret = self.run_cli_command('service listgroups')
        self.assertEqual(ret, 0)
        mock_service_get_all.assert_called_once_with()

    @mock.patch('kolla_cli.api.group.Group.add_service')
    @mock.patch('kolla_cli.api.client.ClientApi.group_get')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_service_addgroup(self, _, mock_group_get,
                              mock_group_add_service):
        servicename = 'service1'
        groupname = 'group1'
        fake_group = self.get_fake_group(groupname)
        mock_group_get.return_value = [fake_group]
        ret = self.run_cli_command('service addgroup %s %s'
                                   % (servicename, groupname))
        self.assertEqual(ret, 0)
        mock_group_get.assert_called_once_with([groupname])
        mock_group_add_service.assert_called_once_with(servicename)

    @mock.patch('kolla_cli.api.group.Group.remove_service')
    @mock.patch('kolla_cli.api.client.ClientApi.group_get')
    @mock.patch('kolla_cli.shell.KollaCli._is_inventory_present',
                return_value=True)
    def test_service_removegroup(self, _, mock_group_get,
                                 mock_group_remove_service):
        servicename = 'service1'
        groupname = 'group1'
        fake_group = self.get_fake_group(groupname,
                                         servicenames=[servicename])
        mock_group_get.return_value = [fake_group]
        ret = self.run_cli_command('service removegroup %s %s'
                                   % (servicename, groupname))
        self.assertEqual(ret, 0)
        mock_group_get.assert_called_once_with([groupname])
        mock_group_remove_service.assert_called_once_with(servicename)



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


class Job(object):
    """Job"""
    def __init__(self, ansible_job):
        self._ansible_job = ansible_job

    def wait(self):
        # type: () -> int
        """Wait for job to complete

        :return: 0 if job succeeded, 1 if job failed
        :rtype: int
        """
        return self._ansible_job.wait()

    def get_status(self):
        # type: () -> int
        """Get status of job

        :return: None: job is still running
                 0: job succeeded
                 1: job failed
                 2: job killed by user
        :rtype: int or None
        """
        return self._ansible_job.get_status()

    def get_error_message(self):
        # type: () -> str
        """Get error message

        :return: if job failed, this will return the error message.
        :rtype: string
        """
        return self._ansible_job.get_error_message()

    def get_console_output(self):
        # type: () -> str
        """Get the console output from the job

        :return: console output useful for debugging failed jobs.
        :rtype: string
        """
        return self._ansible_job.get_command_output()

    def kill(self):
        """kill the job"""
        self._ansible_job.kill()



#!/usr/bin/env python
#
# Copyright 2018 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import os
import sys

from kolla_cli.common import utils

kolla_ansible_source_base = '../kolla-ansible'
kolla_ansible_home_target = utils.get_kolla_ansible_home()
kolla_ansible_etc_target = utils.get_kolla_etc()
ansible = 'ansible'
kolla = 'kolla'
kolla_cli = 'kolla-cli'
tools = 'tools'


def setup_ansible_etc():
    # if the kolla-cli directory for the inventory doesn't exist
    # already then make it. this will also create the directory the
    # globals and password file goes into
    cli_etc_dir = os.path.join(kolla_ansible_etc_target,
                               kolla_cli, ansible)
    if not os.path.exists(cli_etc_dir):
        make_cli_etc_dir_cmd = ('mkdir -p %s' % cli_etc_dir)
        _command_exec(make_cli_etc_dir_cmd)

    # Create the inventory file (if it doesn't already exist).
    # (The script will exit here if the user doesn't have sufficient privs.)
    inventory_file_path = os.path.join(cli_etc_dir, 'inventory.json')
    if not os.path.exists(inventory_file_path):
        touch_inventory_file_cmd = ('touch %s' % inventory_file_path)
        _command_exec(touch_inventory_file_cmd)

    # copy over all kolla ansible etc files
    kolla_ansible_etc_source = os.path.join(kolla_ansible_source_base,
                                            'etc', kolla)
    for etc_file in os.listdir(kolla_ansible_etc_source):
        if not os.path.exists(os.path.join(kolla_ansible_etc_target,
                                           etc_file)):
            copy_kolla_etc_files_cmd = (
                'cp -a {source_dir}/{filename} {target_dir}'.format(
                    source_dir=kolla_ansible_etc_source,
                    filename=etc_file,
                    target_dir=kolla_ansible_etc_target))
            _command_exec(copy_kolla_etc_files_cmd)

    # add ssh keys for cli
    key_path = os.path.join(os.getenv('HOME'), '.ssh', 'id_rsa')
    if not os.path.exists(key_path):
        # generate new ssh keys
        keygen_cmd = 'ssh-keygen -t rsa -N \'\' -f %s' % key_path
        _command_exec(keygen_cmd)
    # copy the public key to where kolla-cli expects it
    pub_key_path = os.path.join(os.getenv('HOME'), '.ssh', 'id_rsa.pub')
    cli_etc_path = os.path.join(kolla_ansible_etc_target, kolla_cli)
    copy_cmd = 'cp -p %s %s/' % (pub_key_path, cli_etc_path)
    _command_exec(copy_cmd)


def setup_ansible_home():
    # make cli home ansible directory
    cli_ansible_dir = os.path.join(kolla_ansible_home_target,
                                   kolla_cli, ansible)
    if not os.path.exists(cli_ansible_dir):
        make_cli_ansible_dir_cmd = ('mkdir -p %s' % cli_ansible_dir)
        _command_exec(make_cli_ansible_dir_cmd)

    # make cli home tools directory
    cli_tools_dir = os.path.join(kolla_ansible_home_target,
                                 kolla_cli, tools)
    if not os.path.exists(cli_tools_dir):
        make_cli_tools_dir_cmd = ('mkdir -p %s' % cli_tools_dir)
        _command_exec(make_cli_tools_dir_cmd)

    # move cli tools files to tools directory
    copy_cli_tools_files_cmd = ('cp -a %s %s' % ('./tools/*', cli_tools_dir))
    _command_exec(copy_cli_tools_files_cmd)

    # create cli ansible lock file
    lock_file_path = os.path.join(kolla_ansible_home_target,
                                  kolla_cli, 'ansible.lock')
    if not os.path.exists(lock_file_path):
        touch_ansible_lock_file_cmd = ('touch %s' % lock_file_path)
        _command_exec(touch_ansible_lock_file_cmd)

    # copy over all kolla ansible home files
    kolla_ansible_home_source = os.path.join(kolla_ansible_source_base,
                                             ansible)
    copy_kolla_home_files_cmd = ('cp -a %s %s' % (kolla_ansible_home_source,
                                                  kolla_ansible_home_target))
    _command_exec(copy_kolla_home_files_cmd)

    # create the host_vars directory if it doesn't exist already
    host_vars_path = os.path.join(kolla_ansible_home_target,
                                  ansible, 'host_vars')
    if not os.path.exists(host_vars_path):
        make_kolla_host_vars_dir_cmd = ('mkdir %s' % host_vars_path)
        _command_exec(make_kolla_host_vars_dir_cmd)

    # make link from etc globals to home globals
    target_etc_path = os.path.join(kolla_ansible_etc_target, 'globals.yml')
    target_home_link = os.path.join(kolla_ansible_home_target,
                                    ansible, 'group_vars', '__GLOBAL__')
    if not os.path.exists(target_home_link):
        link_globals_file_cmd = ('ln -s %s %s' % (target_etc_path,
                                                  target_home_link))
        _command_exec(link_globals_file_cmd)


def _command_exec(command):
    print('running - %s' % command)
    error, _ = utils.run_cmd(command)
    if error:
        print('error - %s' % error)
        sys.exit(1)


def main():
    """make sure kolla-ansible and cli files are in the right places"""

    if len(sys.argv) >= 2:
        global kolla_ansible_source_base
        kolla_ansible_source_base = sys.argv[1]

    setup_ansible_etc()
    setup_ansible_home()


if __name__ == '__main__':
    main()



# Copyright(c) 2016, Oracle and/or its affiliates.  All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

from kolla_cli.api.exceptions import InvalidArgument
from kolla_cli.common.support import dump
from kolla_cli.common.support import get_logs
from kolla_cli.common.utils import check_arg
from kolla_cli.common.utils import safe_decode
import kolla_cli.i18n as u

MYPY = False
if MYPY:
    from typing import List  # noqa


class SupportApi(object):

    def support_dump(self, dirpath):
        # type: (str) -> str
        """Dumps configuration data for debugging.

        Dumps most files in /etc/kolla and /usr/share/kolla into a
        tar file so be given to support / development to help with
        debugging problems.

        :param dirpath: path to directory where dump will be placed
        :type dirpath: string
        :return: path to dump file
        :rtype: string
        """
        check_arg(dirpath, u._('Directory path'), str)
        dirpath = safe_decode(dirpath)
        if not os.path.exists(dirpath):
            raise InvalidArgument(u._('Directory path: {path} does not exist')
                                  .format(path=dirpath))
        dumpfile_path = dump(dirpath)
        return dumpfile_path

    def support_get_logs(self, servicenames, hostname, dirpath):
        # type: (List[str], str, str) -> None
        """get container logs

        Fetch the container log files of services from the specified hosts.
        The log files will be placed in the named directory. All the containers
        for the host will be placed in a directory named hostname. The file
        names for each log will be servicename_id.log.

        :param servicenames: names of services (ie nova, glance, etc)
        :type servicenames: list of strings
        :param hostname: name of host to look for logs on
        :type hostname: string
        :param dirpath: path of directory where log files will be written
        :type dirpath: string
        """
        check_arg(dirpath, u._('Directory path'), str)
        dirpath = safe_decode(dirpath)
        if not os.path.exists(dirpath):
            raise InvalidArgument(u._('Directory path: {path} does not exist')
                                  .format(path=dirpath))

        check_arg(servicenames, u._('Service names'), list)
        servicenames = safe_decode(servicenames)
        check_arg(hostname, u._('Host names'), str)
        hostname = safe_decode(hostname)

        get_logs(servicenames, hostname, dirpath)
